!*****************************************************************************!
!* 
!*      $RCSfile: CreateMesh.f90,v $
!*
!*      $Revision: 1.6 $
!*
!*      $Date: 2004/02/09 12:00:23 $
!*
!*      $Author: roller $
!*
!*****************************************************************************!

MODULE CreateMesh_mod
  !---------------------------------------------------------------------------!
  IMPLICIT NONE
  PRIVATE
  !---------------------------------------------------------------------------!
  INTERFACE CreateMesh
     MODULE PROCEDURE CreateMesh
  END INTERFACE
  !---------------------------------------------------------------------------!
  PUBLIC  :: CreateMesh
  !---------------------------------------------------------------------------!

CONTAINS

  SUBROUTINE CreateMesh(Const,Mesh,MPI)
    USE Types
    !---------------------------------------------------------------------------!
    IMPLICIT NONE
    !---------------------------------------------------------------------------!
    ! Argument list declaration
    TYPE(tConstants)        :: Const
    TYPE(tMesh)             :: Mesh
    TYPE(tMPI)              :: MPI
    ! Local variable declaration
    INTEGER                 :: i,j
    REAL                    :: dx, dy
    !---------------------------------------------------------------------------!
    
    if (MPI%myrank == 0) then
       WRITE(*,*) 'Creating Mesh...'
    endif

    ! Computing cell size
    dx = Mesh%xlen/REAL(MPI%IMAX)
    dy = Mesh%ylen/REAL(MPI%JMAX)
    Mesh%dx = dx
    Mesh%dy = dy
 
    if (MPI%myrank == 0) then
       WRITE(*,*) '   Cellsize: dx = ', dx, 'dy = ', dy
    endif

    ! Normal vectors
    Mesh%xNormal(:) = (/1.0, 0.0/)
    Mesh%yNormal(:) = (/0.0, 1.0/)

    ! Setting barycenters and cell surfaces
    DO i = 0, Mesh%IMAX+1
       DO j=0, Mesh%JMAX+1
          Mesh%Bary(i,j,1) = Mesh%x0 + dx/2. + (i-1 + MPI%xOffset)*dx
          Mesh%Bary(i,j,2) = Mesh%y0 + dy/2. + (j-1 + MPI%yOffset)*dy
          Mesh%Surface(i,j) = dx*dy
       ENDDO
    ENDDO

    SELECT CASE (Const%icType)
    CASE (5,6,7,8,9)
       Mesh%lmin = Mesh%dx                   ! Minimum length for 1D calculation
    CASE DEFAULT
       Mesh%lmin = 0.5*MIN(Mesh%dx,Mesh%dy)  ! Minimum length for 2D calculation
    ENDSELECT


  END SUBROUTINE CreateMesh


END MODULE CreateMesh_mod
