!*****************************************************************************!
!* 
!*      $RCSfile: DataOutput.f90,v $
!*
!*      $Revision: 1.14 $
!*
!*      $Date: 2004/02/09 12:07:58 $
!*
!*      $Author: roller $
!*
!*****************************************************************************!

MODULE DataOutput_mod
  !---------------------------------------------------------------------------!
  IMPLICIT NONE
  PRIVATE
  !---------------------------------------------------------------------------!
  INTERFACE DataOutput
     MODULE PROCEDURE DataOutput
  END INTERFACE
  !---------------------------------------------------------------------------!
  PUBLIC  :: DataOutput
  !---------------------------------------------------------------------------!

CONTAINS

  SUBROUTINE DataOutput(iter,dt,time,printtime,pvar,Const,Mesh,FileIO,MPI,iExStat)
    USE Types
    !---------------------------------------------------------------------------!
    IMPLICIT NONE
    !---------------------------------------------------------------------------!
    ! Argument list declaration
    TYPE(tConstants)        :: Const
    TYPE(tMesh)             :: Mesh
    TYPE(tFileIO)           :: FileIO
    TYPE(tMPI)              :: MPI
    INTEGER                 :: iter
    INTEGER, OPTIONAL       :: iExStat          ! Flag for exact solution (0 / 1)
    REAL                    :: dt
    REAL                    :: time
    REAL                    :: printtime
    REAL                    :: pvar(0:Mesh%IMAX+1,0:Mesh%JMAX+1,Const%nvar)
    ! Local variable declaration
    CHARACTER(LEN=120)      :: filestring
    CHARACTER(LEN=120)      :: filestring2
    CHARACTER(LEN=40)       :: citer
    CHARACTER(LEN=4)        :: mpirank
    CHARACTER(LEN=4)        :: citerNr
    INTEGER                 :: i,j
    INTEGER                 :: ntime
    REAL                    :: x
    INTEGER                 :: stat
    !---------------------------------------------------------------------------!

    WRITE(citer,'(I9.9)') iter
    write(mpirank,'(I4.4)') MPI%myrank

    ! wievielter Ausgabezeitpunkt bin ich?
    write(citerNr,105) INT( printtime/FileIO%iOutput + 0.5)

    IF( .NOT.PRESENT(iExStat).OR.(iExStat.EQ.0) ) THEN
       if (MPI%myrank == 0) then
          WRITE(*,100) '-- Iteration ', TRIM(citer), '  dt = ', dt, &
                       '   Current Time: ', time
       endif
    endif

100 format (a,a,a,f10.7,a,f10.5)

    ! Preparing file access
    WRITE(citer,'(I9.9)') iter
    IF( PRESENT(iExStat).AND.(iExStat.EQ.1) ) THEN
       IF(FileIO%iVisuProg.EQ.1) THEN
          filestring  = TRIM(FileIO%strOutFile) // '_ex' // '-' // TRIM(citerNr) // '-' // TRIM(mpirank) // '.dat'
       ELSEIF(FileIO%iVisuProg.EQ.2) THEN
          filestring  = TRIM(FileIO%strOutFile) // '_ex' // '-' // TRIM(citerNr) // '-' // TRIM(mpirank) // '.dx'
          filestring2 = TRIM(FileIO%strOutFile) // '_ex' // '-' // TRIM(mpirank) // '.dx'
       ENDIF
    ELSE
       IF(FileIO%iVisuProg.EQ.1) THEN
          filestring  = TRIM(FileIO%strOutFile) // '-' // TRIM(citerNr) // '-' // TRIM(mpirank) // '.dat'
       ELSEIF(FileIO%iVisuProg.EQ.2) THEN
          filestring  = TRIM(FileIO%strOutFile) // '-' // TRIM(citerNr) // '-' // TRIM(mpirank) // '.dx'
          filestring2 = TRIM(FileIO%strOutFile) // '-' // TRIM(mpirank) // '.dx'
       ENDIF
    ENDIF
    !
    ! open file for single timestep
    !
    OPEN(UNIT   = FileIO%uOutFile, &
         FILE   = filestring     , &
         STATUS = 'UNKNOWN'      , &
         RECL   = 700            , &
         IOSTAT = stat      )
    !
    ! OpenDx: to create a movie, all timesteps data have to be in one file
    !
    IF(FileIO%iVisuProg.EQ.2) THEN
       if (time .eq. 0.0) then
          ! open file for all timestep data - NEW FILE -
          OPEN(UNIT     = FileIO%uOutFile +1, &
               FILE     = filestring2     , &
               STATUS   = 'UNKNOWN'      , &
               RECL     = 700            , &
               IOSTAT   = stat      )
       else
          ! open file for all timestep data - APPEND -
          OPEN(UNIT     = FileIO%uOutFile +1, &
               FILE     = filestring2     , &
               STATUS   = 'UNKNOWN'      , &
               POSITION = 'APPEND'      , &
               RECL     = 700            , &
               IOSTAT   = stat      )
       endif
    ENDIF

    SELECT CASE(FileIO%iVisuProg)
    CASE(1) ! Tecplot
!!$       if (MPI%myrank == 0) then
          WRITE(FileIO%uOutFile,*) 'TITLE="Time ',time,', Zeitschritt ',iter,' " '
          WRITE(FileIO%uOutFile,*) 'VARIABLES = "x","y","rho","u","v","p" '
          WRITE(FileIO%uOutFile,*) 'ZONE I=',Mesh%IMAX,'J=',Mesh%JMAX,' F=POINT'
!!$       endif
       DO j=1,Mesh%JMAX
          DO i=1,Mesh%IMAX
             WRITE(FileIO%uOutFile,*) &
                  Mesh%Bary(i,j,1), &
                  Mesh%Bary(i,j,2), &
                  pvar(i,j,1),      &
                  pvar(i,j,2),      &
                  pvar(i,j,3),      &
                  pvar(i,j,4)
          ENDDO
       ENDDO
    CASE(2) ! IBM Open DX
       ! Filter tiny values
       WHERE(ABS(pvar(:,:,:)).LE.1e-40) 
          pvar(:,:,:) = 0.
       END WHERE

       SELECT CASE(Const%icType)
       CASE(5,6,7,8,9)
          ! 1D case
          ! ----------------------------------------------------------------
          ! Writing position header
          ! ----------------------------------------------------------------
          WRITE(FileIO%uOutFile,*) '# Time = ',time,', Zeitschritt ',iter
          WRITE(FileIO%uOutFile,*) '# The irregular positions, which are 1D points'
          WRITE(FileIO%uOutFile,*) 'object 1 class array type float rank 1 shape 1 items ', &
               Mesh%IMAX,' data follows'
          WRITE(FileIO%uOutFile,*) Mesh%x0+Mesh%dx/2.
          x=Mesh%x0+Mesh%dx/2.
          DO i=1,Mesh%IMAX-1
             x=x+Mesh%dx
             WRITE(FileIO%uOutFile,*) x
          ENDDO
          ! ----------------------------------------------------------------
          ! Writing connection header
          ! ----------------------------------------------------------------
          WRITE(FileIO%uOutFile,*) '# The irregular connections, which are lines'
          WRITE(FileIO%uOutFile,*) 'object 2 class array type int rank 1 shape 2 items ', &
               Mesh%IMAX-1,'  data follows'
          DO i=1,Mesh%IMAX-1
             WRITE(FileIO%uOutFile,*) i-1, i
          ENDDO
          WRITE(FileIO%uOutFile,*) 'attribute "element type" string "lines" '
          WRITE(FileIO%uOutFile,*) 'attribute "ref" string "positions" '
          ! ----------------------------------------------------------------
          ! Write density data header
          ! ----------------------------------------------------------------
          WRITE(FileIO%uOutFile,*) '#The density data' 
          WRITE(FileIO%uOutFile,*) 'object 3 class array type float rank 0 items ', &
               Mesh%IMAX, ' data follows'
          ! Write data
          DO i=1, Mesh%IMAX
             WRITE(FileIO%uOutFile,*) pvar(i,1,1)
          ENDDO
          ! Write data attributes
          WRITE(FileIO%uOutFile,*) 'attribute "dep" string "positions"'
          ! ----------------------------------------------------------------
          ! Write x-velocity data header
          ! ----------------------------------------------------------------
          WRITE(FileIO%uOutFile,*) '#The velocity data' 
          WRITE(FileIO%uOutFile,*)  'object 4 class array type float rank 0 items ', &
               Mesh%IMAX, ' data follows'
          ! Write data
          DO i=1, Mesh%IMAX
             WRITE(FileIO%uOutFile,*) pvar(i,1,2)
          ENDDO
          ! Write data attributes
          WRITE(FileIO%uOutFile,*) 'attribute "dep" string "positions"'
          ! ----------------------------------------------------------------
          ! Write pressure data header
          ! ----------------------------------------------------------------
          WRITE(FileIO%uOutFile,*) '#The pressure data' 
          WRITE(FileIO%uOutFile,*) 'object 5 class array type float rank 0 items ', &
               Mesh%IMAX, ' data follows'
          ! Write data
          DO i=1, Mesh%IMAX
             WRITE(FileIO%uOutFile,*) pvar(i,1,4)
          ENDDO
          ! Write data attributes
          WRITE(FileIO%uOutFile,*) 'attribute "dep" string "positions"'
          ! ----------------------------------------------------------------
          ! Build field structure
          ! ----------------------------------------------------------------
          WRITE(FileIO%uOutFile,*) '#The density field' 
          WRITE(FileIO%uOutFile,*) 'object 100 class field'
          WRITE(FileIO%uOutFile,*) 'component "positions" value 1'
          WRITE(FileIO%uOutFile,*) 'component "connections" value 2'
          WRITE(FileIO%uOutFile,*) 'component "data" value 3'
          WRITE(FileIO%uOutFile,*) 'attribute "name" string "rho"'
          WRITE(FileIO%uOutFile,*) '# The velocity field'
          WRITE(FileIO%uOutFile,*) 'object 200 class field'
          WRITE(FileIO%uOutFile,*) 'component "positions" value 1'
          WRITE(FileIO%uOutFile,*) 'component "connections" value 2'
          WRITE(FileIO%uOutFile,*) 'component "data" value 4'
          WRITE(FileIO%uOutFile,*) 'attribute "name" string "V"'
          WRITE(FileIO%uOutFile,*) '# The pressure field'
          WRITE(FileIO%uOutFile,*) 'object 300 class field'
          WRITE(FileIO%uOutFile,*) 'component "positions" value 1'
          WRITE(FileIO%uOutFile,*) 'component "connections" value 2'
          WRITE(FileIO%uOutFile,*) 'component "data" value 5'
          WRITE(FileIO%uOutFile,*) 'attribute "name" string "p"'
          WRITE(FileIO%uOutFile,*) 'object "default" class group'
          WRITE(FileIO%uOutFile,*) 'member "rho" value 100'
          WRITE(FileIO%uOutFile,*) 'member "u"   value 200'
          WRITE(FileIO%uOutFile,*) 'member "p"   value 300'
          WRITE(FileIO%uOutFile,*) 'end'
       CASE DEFAULT
          ! 2D case - single timestep file
          ! ----------------------------------------------------------------
          ! Writing position header
          ! ----------------------------------------------------------------
          WRITE(FileIO%uOutFile,*) '# The regular positions'
          WRITE(FileIO%uOutFile,120) &
               'object "grid"  class gridpositions counts ', & 
               Mesh%IMAX, ' ', Mesh%JMAX 
          WRITE(FileIO%uOutFile,110) 'origin          ', Mesh%x0, Mesh%y0
          WRITE(FileIO%uOutFile,110) 'delta           ', Mesh%dx, 0.
          WRITE(FileIO%uOutFile,110) 'delta           ', 0.,      Mesh%dy
          ! ----------------------------------------------------------------
          ! Writing connection header
          ! ----------------------------------------------------------------
          WRITE(FileIO%uOutFile,*) '# The regular connections'
          WRITE(FileIO%uOutFile,120) &
               'object "c_list" class gridconnections counts ', &  
               Mesh%IMAX, ' ', Mesh%JMAX
          ! ----------------------------------------------------------------
          ! Write density data header
          ! ----------------------------------------------------------------
          WRITE(FileIO%uOutFile,*) '#The density data' 
          WRITE(FileIO%uOutFile,*) 'object "rv_',trim(citerNr),&
               '" class array type float rank 0 items ', & 
               Mesh%IMAX*Mesh%JMAX, ' data follows'
          ! Write data
          DO i=1, Mesh%IMAX
             DO j=1, Mesh%JMAX
                WRITE(FileIO%uOutFile,*) pvar(i,j,1)
             ENDDO
          ENDDO
          ! Write data attributes
          WRITE(FileIO%uOutFile,*) 'attribute "dep" string "positions"'
          ! ----------------------------------------------------------------
          ! Write velocity data header
          ! ----------------------------------------------------------------
          WRITE(FileIO%uOutFile,*) '#The velocity data' 
          WRITE(FileIO%uOutFile,*)  'object "Vv_',trim(citerNr),&
               '" class array type float rank 1 shape 2 items ', &
               Mesh%IMAX*Mesh%JMAX, ' data follows'
          ! Write data
          DO i=1, Mesh%IMAX
             DO j=1, Mesh%JMAX
                WRITE(FileIO%uOutFile,*) pvar(i,j,2), pvar(i,j,3)
             ENDDO
          ENDDO
          ! Write data attributes
          WRITE(FileIO%uOutFile,*) 'attribute "dep" string "positions"'
          ! ----------------------------------------------------------------
          ! Write pressure data header
          ! ----------------------------------------------------------------
          WRITE(FileIO%uOutFile,*) '#The pressure data' 
          WRITE(FileIO%uOutFile,*) 'object "pv_',trim(citerNr),&
               '" class array type float rank 0 items ', &
               Mesh%IMAX*Mesh%JMAX, ' data follows'
          ! Write data
          DO i=1, Mesh%IMAX
             DO j=1, Mesh%JMAX
                WRITE(FileIO%uOutFile,*) pvar(i,j,4)
             ENDDO
          ENDDO
          ! Write data attributes
          WRITE(FileIO%uOutFile,*) 'attribute "dep" string "positions"'
          ! ----------------------------------------------------------------
          ! Build field structure
          ! ----------------------------------------------------------------
          WRITE(FileIO%uOutFile,*) '#The density field' 
          WRITE(FileIO%uOutFile,*) 'object "rho_',trim(citerNr),'" class field'
          WRITE(FileIO%uOutFile,*) 'component "positions" value "grid"'
          WRITE(FileIO%uOutFile,*) 'component "connections" value "c_list"'
          WRITE(FileIO%uOutFile,*) 'component "data" value "rv_',trim(citerNr),'"'
          WRITE(FileIO%uOutFile,*) 'attribute "name" string "rho"'
          WRITE(FileIO%uOutFile,*) '# The velocity field'
          WRITE(FileIO%uOutFile,*) 'object "V_',trim(citerNr),'" class field'
          WRITE(FileIO%uOutFile,*) 'component "positions" value "grid"'
          WRITE(FileIO%uOutFile,*) 'component "connections" value "c_list"'
          WRITE(FileIO%uOutFile,*) 'component "data" value "Vv_',trim(citerNr),'"'
          WRITE(FileIO%uOutFile,*) 'attribute "name" string "V"'
          WRITE(FileIO%uOutFile,*) '# The pressure field'
          WRITE(FileIO%uOutFile,*) 'object "p_',trim(citerNr),'" class field'
          WRITE(FileIO%uOutFile,*) 'component "positions" value "grid"'
          WRITE(FileIO%uOutFile,*) 'component "connections" value "c_list"'
          WRITE(FileIO%uOutFile,*) 'component "data" value "pv_',trim(citerNr),'"'
          WRITE(FileIO%uOutFile,*) 'attribute "name" string "p"'
          WRITE(FileIO%uOutFile,*) 'object "default" class group'
          WRITE(FileIO%uOutFile,*) 'member "rho" value "rho_',trim(citerNr),'"'
          WRITE(FileIO%uOutFile,*) 'member "v"   value "V_',trim(citerNr),'"'
          WRITE(FileIO%uOutFile,*) 'member "p"   "p_',trim(citerNr),'"'
          WRITE(FileIO%uOutFile,*) 'end'

          !
          ! 2D case - all timesteps in one file  
          !
          !
          ! 1. Zeitschritt: Header schreiben
          if (time .eq. 0.0) then
             ! ----------------------------------------------------------------
             ! Writing position header
             ! ----------------------------------------------------------------
             WRITE(FileIO%uOutFile+1,*) '# The regular positions'
             WRITE(FileIO%uOutFile+1,120) 'object "grid"  class gridpositions counts ', &
                  Mesh%IMAX, ' ', Mesh%JMAX 
             WRITE(FileIO%uOutFile+1,110) 'origin          ', Mesh%x0, Mesh%y0
             WRITE(FileIO%uOutFile+1,110) 'delta           ', Mesh%dx, 0.
             WRITE(FileIO%uOutFile+1,110) 'delta           ', 0.,      Mesh%dy
             ! ----------------------------------------------------------------
             ! Writing connection header
             ! ----------------------------------------------------------------
             WRITE(FileIO%uOutFile+1,*) '# The regular connections'
             WRITE(FileIO%uOutFile+1,120) &
                  'object "c_list" class gridconnections counts ', &  
                  Mesh%IMAX, ' ', Mesh%JMAX
          endif

          write(FileIO%uOutFile+1,*) '#Timestep Nr.',citerNr, & 
               ', time= ',time, 'Iterations= ',iter
          ! ----------------------------------------------------------------
          ! Write density data header
          ! ----------------------------------------------------------------
          WRITE(FileIO%uOutFile+1,*) '#The density data' 
          WRITE(FileIO%uOutFile+1,*) 'object "rv_',trim(citerNr),&
               '" class array type float rank 0 items ', & 
               Mesh%IMAX*Mesh%JMAX, ' data follows'
          ! Write data
          DO i=1, Mesh%IMAX
             DO j=1, Mesh%JMAX
                WRITE(FileIO%uOutFile+1,*) pvar(i,j,1)
             ENDDO
          ENDDO
          ! Write data attributes
          WRITE(FileIO%uOutFile+1,*) 'attribute "dep" string "positions"'
          ! ----------------------------------------------------------------
          ! Write velocity data header
          ! ----------------------------------------------------------------
          WRITE(FileIO%uOutFile+1,*) '#The velocity data' 
          WRITE(FileIO%uOutFile+1,*)  'object "Vv_',trim(citerNr),&
               '" class array type float rank 1 shape 2 items ', &
               Mesh%IMAX*Mesh%JMAX, ' data follows'
          ! Write data
          DO i=1, Mesh%IMAX
             DO j=1, Mesh%JMAX
                WRITE(FileIO%uOutFile+1,*) pvar(i,j,2), pvar(i,j,3)
             ENDDO
          ENDDO
          ! Write data attributes
          WRITE(FileIO%uOutFile+1,*) 'attribute "dep" string "positions"'
          ! ----------------------------------------------------------------
          ! Write pressure data header
          ! ----------------------------------------------------------------
          WRITE(FileIO%uOutFile+1,*) '#The pressure data' 
          WRITE(FileIO%uOutFile+1,*) 'object "pv_',trim(citerNr),&
               '" class array type float rank 0 items ', &
               Mesh%IMAX*Mesh%JMAX, ' data follows'
          ! Write data
          DO i=1, Mesh%IMAX
             DO j=1, Mesh%JMAX
                WRITE(FileIO%uOutFile+1,*) pvar(i,j,4)
             ENDDO
          ENDDO
          ! Write data attributes
          WRITE(FileIO%uOutFile+1,*) 'attribute "dep" string "positions"'
          ! ----------------------------------------------------------------
          ! Build field structure
          ! ----------------------------------------------------------------
          WRITE(FileIO%uOutFile+1,*) '#The density field' 
          WRITE(FileIO%uOutFile+1,*) 'object "rho_',trim(citerNr),'" class field'
          WRITE(FileIO%uOutFile+1,*) 'component "positions" value "grid"'
          WRITE(FileIO%uOutFile+1,*) 'component "connections" value "c_list"'
          WRITE(FileIO%uOutFile+1,*) 'component "data" value "rv_',trim(citerNr),'"'
          WRITE(FileIO%uOutFile+1,*) 'attribute "name" string "rho"'
          WRITE(FileIO%uOutFile+1,*) '# The velocity field'
          WRITE(FileIO%uOutFile+1,*) 'object "V_',trim(citerNr),'" class field'
          WRITE(FileIO%uOutFile+1,*) 'component "positions" value "grid"'
          WRITE(FileIO%uOutFile+1,*) 'component "connections" value "c_list"'
          WRITE(FileIO%uOutFile+1,*) 'component "data" value "Vv_',trim(citerNr),'"'
          WRITE(FileIO%uOutFile+1,*) 'attribute "name" string "V"'
          WRITE(FileIO%uOutFile+1,*) '# The pressure field'
          WRITE(FileIO%uOutFile+1,*) 'object "p_',trim(citerNr),'" class field'
          WRITE(FileIO%uOutFile+1,*) 'component "positions" value "grid"'
          WRITE(FileIO%uOutFile+1,*) 'component "connections" value "c_list"'
          WRITE(FileIO%uOutFile+1,*) 'component "data" value "pv_',trim(citerNr),'"'
          WRITE(FileIO%uOutFile+1,*) 'attribute "name" string "p"'

          if (printtime .eq. Const%Stoptime) then
             ! 
             ! Letzter Zeitschritt: Tail schreiben
             !
             ntime=INT(Const%Stoptime / FileIO%iOutput + 0.5) 

             WRITE(FileIO%uOutFile+1,*) 'object "rho" class series'
             do i=0,ntime
                WRITE(citerNr,105) i
                WRITE(FileIO%uOutFile+1,*) 'member ',i,' value "rho_',trim(citerNr),'"'
             enddo
             WRITE(FileIO%uOutFile+1,*) 'attribute "name" string "rho"'
             
             WRITE(FileIO%uOutFile+1,*) 'object "V" class series'
             do i=0,ntime
                WRITE(citerNr,'(I4.4)') i
                WRITE(FileIO%uOutFile+1,*) 'member ',i,' value "V_',trim(citerNr),'"'
             ENDDO
             WRITE(FileIO%uOutFile+1,*) 'attribute "name" string "V"'

             WRITE(FileIO%uOutFile+1,*) 'object "p" class series'
             do i=0,ntime
                WRITE(citerNr,'(I4.4)') i
                WRITE(FileIO%uOutFile+1,*) 'member ',i,' value "p_',trim(citerNr),'"'
             ENDDO
             WRITE(FileIO%uOutFile+1,*) 'attribute "name" string "p"'
             WRITE(FileIO%uOutFile+1,*) 'object "default" class group'

             WRITE(FileIO%uOutFile+1,*) 'member "rho" value "rho"'
             WRITE(FileIO%uOutFile+1,*) 'member "V"   value "V"'
             WRITE(FileIO%uOutFile+1,*) 'member "p"   value "p"'
             
             WRITE(FileIO%uOutFile+1,*) 'end'
          endif
       END SELECT
       
    END SELECT
    
    CLOSE(FileIO%uOutFile)
    CLOSE(FileIO%uOutFile+1)
    
105 format (i4.4)
110 format (a,F15.10,F15.10)
120 format (a,i4,a,i4)
    
    IF( .NOT.PRESENT(iExStat).OR.(iExStat.EQ.0) ) THEN
       printtime = printtime + FileIO%iOutput
    ENDIF
  END SUBROUTINE DataOutput


END MODULE DataOutput_mod
