!*****************************************************************************!
!* 
!*      $RCSfile: ExactSolution.f90,v $
!*
!*      $Revision: 1.7 $
!*
!*      $Date: 2004/02/09 11:57:22 $
!*
!*      $Author: roller $
!*
!*****************************************************************************!

MODULE ExactSolution_mod
  !----------------------------------------------------------------------------
  IMPLICIT NONE
  PRIVATE
  !----------------------------------------------------------------------------
  INTERFACE ExactSolution
     MODULE PROCEDURE ExactSolution
  END INTERFACE
  INTERFACE IniExactSolution
     MODULE PROCEDURE IniExactSolution
  END INTERFACE
  INTERFACE CloseExactSolution
     MODULE PROCEDURE CloseExactSolution
  END INTERFACE
  REAL, POINTER               :: pvar_ex(:,:,:)     ! Exact Solution (PrimVar)!

  !----------------------------------------------------------------------------
  PUBLIC  :: ExactSolution,IniExactSolution,CloseExactSolution
  !----------------------------------------------------------------------------

CONTAINS

  SUBROUTINE IniExactSolution(pvar,Const,Mesh,FileIO,MPI)
    USE Types
    USE DataOutput_mod
    !---------------------------------------------------------------------------!
    IMPLICIT NONE
    !---------------------------------------------------------------------------!
    ! Argument list declaration
    TYPE(tConstants)        :: Const
    TYPE(tMesh)             :: Mesh
    TYPE(tFileIO)           :: FileIO
    TYPE(tMPI)              :: MPI
    REAL                    :: pvar(0:Mesh%IMAX+1,0:Mesh%JMAX+1,Const%nvar)
    ! Local variable declaration
    REAL                    :: dummy
    INTEGER                 :: allocStat
    !---------------------------------------------------------------------------!

    ALLOCATE(pvar_ex(0:Mesh%IMAX+1,0:Mesh%JMAX+1,Const%nvar),  &
             STAT = allocStat )

    IF(allocStat.NE.0) THEN
       PRINT *, 'ERROR iniExactSolution: Could not allocate all variables. '
       STOP
    ENDIF

    pvar_ex(:,:,:) = pvar(:,:,:)

    call DataOutput(0,0.,0.,dummy,pvar_ex,Const,Mesh,FileIO,MPI,1)

  END SUBROUTINE IniExactSolution

  SUBROUTINE CloseExactSolution()
    USE Types
    !---------------------------------------------------------------------------!
    IMPLICIT NONE
    !---------------------------------------------------------------------------!
    ! Argument list declaration
    !---------------------------------------------------------------------------!

    DEALLOCATE(pvar_ex)

  END SUBROUTINE CloseExactSolution

  SUBROUTINE ExactSolution(iter,dt,time,Const,Mesh,FileIO,MPI)
    USE Types
    USE SetBoundaryCondition_mod
    USE exact_riemann_mod
    USE DataOutput_mod
    !-------------------------------------------------------
    IMPLICIT NONE
    !-------------------------------------------------------
    ! Argument list declaration
    TYPE(tConstants)        :: Const
    TYPE(tMesh)             :: Mesh
    TYPE(tFileIO)           :: FileIO
    TYPE(tMPI)              :: MPI
    INTEGER                 :: iter
    REAL                    :: dt
    REAL                    :: time
    ! Local variable declaration
    REAL                    :: rho_l, rho_r
    REAL                    :: vn_l , vn_r
    REAL                    :: p_l  , p_r
    REAL                    :: c_l,c_r
    REAL                    :: dummy
    INTEGER                 :: i,j
    !--------------------------------------------------------------------------
    INTENT(IN)              :: time,Const,Mesh
    !--------------------------------------------------------------------------
  

    SELECT CASE(Const%icType)
    CASE(5)
       rho_l=1.0
       rho_r=0.125
       vn_l =0.
       vn_r =0.
       p_l  =1.
       p_r  =0.1

    CASE(6)
       rho_l=1.0
       rho_r=1.0
       vn_l =0.
       vn_r =0.
       p_l  =1000.
       p_r  =0.01

    CASE(7)
       rho_l=1.0
       rho_r=1.0
       vn_l =0.
       vn_r =0.
       p_l  =0.01
       p_r  =100.

    CASE(8)
       rho_l=1.0
       rho_r=0.125
       vn_l =0.75
       vn_r =0.
       p_l  =1.
       p_r  =0.1

    CASE(9)
       rho_l=601./106.
       rho_r=10.
       vn_l =sqrt(9801./1202.)-1.
       vn_r =-1.
       p_l  =10.
       p_r  =0.1

    CASE DEFAULT
       PRINT *, 'ERROR InitialCondition for exact solution not known!'
       STOP
    END SELECT

    c_l = SQRT(Const%gamma*p_l/rho_l)
    c_r = SQRT(Const%gamma*p_r/rho_r)

    do j=1,Mesh%JMAX
       do i=1,Mesh%IMAX

          CALL exact_riemann(Const%gamma,                &
                       rho_l,rho_r,pvar_ex(i,j,1)    ,   &
                       vn_l ,vn_r ,pvar_ex(i,j,2)    ,   &
                       p_l  ,p_r  ,pvar_ex(i,j,4)    ,   &
                       c_l  ,c_r  ,(Mesh%Bary(i,j,1)-Mesh%x0-Mesh%xlen*0.5)/time )
       enddo
    enddo
    
    pvar_ex(:,:,3)=0.
    
    call DataOutput(iter,dt,time,dummy,pvar_ex,Const,Mesh,FileIO,MPI,1)

  END SUBROUTINE ExactSolution

END MODULE ExactSolution_mod
