!*****************************************************************************!
!* 
!*      $RCSfile: InitialCondition.f90,v $
!*
!*      $Revision: 1.11 $
!*
!*      $Date: 2004/02/09 11:57:22 $
!*
!*      $Author: roller $
!*
!*****************************************************************************!

MODULE InitialCondition_mod
  !---------------------------------------------------------------------------!
  IMPLICIT NONE
  PRIVATE
  !---------------------------------------------------------------------------!
  INTERFACE InitialCondition
     MODULE PROCEDURE InitialCondition
  END INTERFACE
  !---------------------------------------------------------------------------!
  PUBLIC  :: InitialCondition
  !---------------------------------------------------------------------------!

CONTAINS

  SUBROUTINE InitialCondition(pvar,Const,Mesh,MPI)
    USE Types
    !---------------------------------------------------------------------------!
    IMPLICIT NONE
    !---------------------------------------------------------------------------!
    ! Argument list declaration
    TYPE(tConstants)        :: Const
    TYPE(tMesh)             :: Mesh
    TYPE(tMPI)              :: MPI
    REAL                    :: pvar(0:Mesh%IMAX+1,0:Mesh%JMAX+1,Const%nvar)
    ! Local variable declaration
    INTEGER                 :: i,j
    REAL                    :: x,y,dr2,amplitude,hwidth
    REAL                    :: l, ly, rhoo, rhos0o, rhos1o, rhoy
    REAL                    :: rho0,p0,m0,u0,rho1,p1,u1
    REAL                    :: po, pso, vso, piql
    REAL                    :: wacon1, wacon2, wacon3
    REAL, pointer           :: phiy(:)
    REAL                    :: RandNr
    !---------------------------------------------------------------------------!

    if (MPI%myrank ==0) WRITE(*,*) 'Setting initial condition...'

    SELECT CASE(Const%icType)
    CASE(1)
       ! Amplitude and halfwidth of Gausspulse
       amplitude = Const%Amplitude
       hwidth    = Const%hwidth
       if (MPI%myrank == 0) then
          WRITE(*,*) '   Initial condition is a 2D Gaussian pressure pulse. '
          WRITE(*,*) '      Amplitude  :  ', amplitude
          WRITE(*,*) '      Half-width :  ', hwidth
       endif
       pvar(:,:,:) = 0.
       pvar(:,:,1) = 1.
       DO i=0,Mesh%IMAX+1
          DO j=0,Mesh%JMAX+1
             x = Mesh%Bary(i,j,1)
             y = Mesh%Bary(i,j,2)
             dr2 = (x-(Mesh%x0+Mesh%xlen/2.))**2 + (y-(Mesh%y0+Mesh%ylen/2.))**2
             pvar(i,j,4) = 1. + amplitude*EXP(-LOG(2.0)*dr2/hwidth**2)
          ENDDO
       ENDDO
    CASE(2)
       ! Amplitude and halfwidth of Gausspulse
       amplitude = Const%Amplitude
       hwidth    = Const%hwidth
       if (MPI%myrank == 0) then
          WRITE(*,*) '   Initial condition is a 1D Gaussian pressure pulse. '
          WRITE(*,*) '      Amplitude  :  ', amplitude
          WRITE(*,*) '      Half-width :  ', hwidth
       endif

       pvar(:,:,:) = 0.
       pvar(:,:,1) = 1.
       DO i=0,Mesh%IMAX+1
          DO j=0,Mesh%JMAX+1
             x = Mesh%Bary(i,j,1)
             y = Mesh%Bary(i,j,2)
             dr2 = (x-(Mesh%x0+Mesh%xlen/2.))**2
             pvar(i,j,4) = 1. + amplitude*EXP(-LOG(2.0)*dr2/hwidth**2)
          ENDDO
       ENDDO
    CASE(3)
       ! Amplitude and halfwidth of Gausspulse
       amplitude = Const%Amplitude
       hwidth    = Const%hwidth
       if (MPI%myrank == 0) then
          WRITE(*,*) '   Initial condition is a 1D Gaussian density pulse. '
          WRITE(*,*) '      Amplitude  :  ', amplitude
          WRITE(*,*) '      Half-width :  ', hwidth
       endif

       pvar(:,:,:) = 0.
       pvar(:,:,2) = 1.
       pvar(:,:,4) = 1.
       DO i=0,Mesh%IMAX+1
          DO j=0,Mesh%JMAX+1
             x = Mesh%Bary(i,j,1)
             y = Mesh%Bary(i,j,2)
             dr2 = (x-(Mesh%x0+Mesh%xlen/2.))**2
             pvar(i,j,1) = 1. + amplitude*EXP(-LOG(2.0)*dr2/hwidth**2)
          ENDDO
       ENDDO
    
    CASE(4)

       if (MPI%myrank == 0) then
          WRITE(*,*) '   Initial condition is causing the ' // &
               'Richtmyer-Meshkov instability. '
       endif
       pvar(:,:,:) = 0.
       pvar(:,:,1) = 1.
       pvar(:,:,4) = 1.

       WHERE ( Mesh%Bary(:,:,1) >= 0.3*Mesh%xlen + &
            (1./30.)*Mesh%xlen*COS(2*Const%Pi*3./Mesh%ylen*Mesh%Bary(:,:,2)) )
          pvar(:,:,1) = 0.25
       END WHERE

       WHERE ( (Mesh%Bary(:,:,1) <= 0.1*Mesh%xlen).AND. &
               (Mesh%Bary(:,:,1) >= (1./30.)*Mesh%xlen)  )
          pvar(:,:,4) = 4.9
          pvar(:,:,1) = 4.22
       END WHERE


    CASE(5)

       if (MPI%myrank == 0) then
          WRITE(*,*) '   Initial condition for standard Sod problem ' 
       endif
       pvar(:,:,:) = 0.
       pvar(:,:,1) = 1.
       pvar(:,:,4) = 1.

       WHERE ( Mesh%Bary(:,:,1) >= Mesh%x0+0.5*Mesh%xlen )
          pvar(:,:,1) = 0.125
          pvar(:,:,4) = 0.1
       END WHERE

    CASE(6)

       if (MPI%myrank == 0) then
          WRITE(*,*) '   Initial condition for Toros test case #3 ' 
       endif
       pvar(:,:,:) = 0.
       pvar(:,:,1) = 1.
       pvar(:,:,4) = 1000.

       WHERE ( Mesh%Bary(:,:,1) >= Mesh%x0+0.5*Mesh%xlen )
          pvar(:,:,4) = 0.01
       END WHERE

    CASE(7)

       if (MPI%myrank == 0) then
          WRITE(*,*) '   Initial condition for Toros test case #4 ' 
       endif
       pvar(:,:,:) = 0.
       pvar(:,:,1) = 1.
       pvar(:,:,4) = 0.01

       WHERE ( Mesh%Bary(:,:,1) >= Mesh%x0+0.5*Mesh%xlen )
          pvar(:,:,4) = 100.
       END WHERE

    CASE(8)

       if (MPI%myrank == 0) then
          WRITE(*,*) '   Initial condition for Toros test case #1 ' 
       endif
       pvar(:,:,1) = 1.
       pvar(:,:,2) = 0.75
       pvar(:,:,3) = 0.
       pvar(:,:,4) = 1.

       WHERE ( Mesh%Bary(:,:,1) >= Mesh%x0+0.5*Mesh%xlen )
          pvar(:,:,1) = 0.125
          pvar(:,:,2) = 0.
          pvar(:,:,4) = 0.1
       END WHERE

    CASE(9)

       if (MPI%myrank == 0) then
          WRITE(*,*) '   Initial condition Shock/Contact Discontinuity interaction' 
       endif
       pvar(:,:,1) = 601./106.
       pvar(:,:,2) = sqrt(9801./1202.)-1.
       pvar(:,:,3) = 0.
       pvar(:,:,4) = 10.

       WHERE ( (Mesh%Bary(:,:,1) >= Mesh%x0+0.2*Mesh%xlen) .and. (Mesh%Bary(:,:,1) < Mesh%x0+0.8*Mesh%xlen) ) 
          pvar(:,:,1) = 1.
          pvar(:,:,2) = -1.
          pvar(:,:,4) = 0.1
       ELSEWHERE ( Mesh%Bary(:,:,1) >= Mesh%x0+0.8*Mesh%xlen )
          pvar(:,:,1) = 10.
          pvar(:,:,2) = -1.
          pvar(:,:,4) = 0.1
       END WHERE

    CASE (10)
       if (MPI%myrank == 0) then
          WRITE(*,*) '   Initial condition Kelvin-Helmholtz instability'
       endif
       !
       ! Hilfsvariablen setzen
       !
       l      = 20.
       ly     = l / 5.
       rhoo   = 1.
       rhos0o = 0.4
       rhos1o = 0.4
       rhoy   = rhos0o / ly
       po     = 1.
       pso    = 2. * Const%gamma
       vso    = sqrt( Const%gamma )
       piql   = Const%Pi / l
       !
       ! haeufig verwendete Werte
       !
       wacon1  =  0.5 / 20. * rhos1o
       wacon2  =  0.5 / 20. * pso
       wacon3  =  0.5 * pso
       !
       ! Allokieren von phi
       !
       allocate(phiy(0:Mesh%JMAX+1))
       !
       ! Berechnung von phi
       !
       do j = 0, Mesh%JMAX+1
          if ( Mesh%Bary(1,j,2) >= -ly .and. Mesh%Bary(1,j,2) <= 0. ) then
          phiy(j) = rhoy * ( Mesh%Bary(1,j,2) + ly )
       elseif ( Mesh%Bary(1,j,2) > 0. .and. Mesh%Bary(1,j,2) <= ly ) then
          phiy(j) = rhoy * ( Mesh%Bary(1,j,2) - ly )
       else
          phiy(j) = 0.
       endif
    enddo
    !
    ! Anfangsgroessen setzen/berechnen
    !
    do j=0,Mesh%JMAX+1
       do i=0,Mesh%IMAX+1
          pvar(i,j,1) = rhoo + wacon1 * ( 1. + cos( piql*Mesh%Bary(i,j,1) ) ) + phiy(j) 
          pvar(i,j,4) = 400. * (po + wacon2 * ( 1. + cos( piql*Mesh%Bary(i,j,1) ) ))
          pvar(i,j,2) = vso * ( 1. + cos( piql*Mesh%Bary(i,j,1) ) )
       enddo
    enddo



 CASE(11)
      
       if (MPI%myrank == 0) then
          WRITE(*,*) '   Initial condition for steady shock problem (x-dir)'
       endif
       rho0 = 1.
       u0   = 1.
       M0   = 20                                 ! Machzahl stromauf
       p0   = 1./const%gamma/M0/M0              ! Druck stromauf
      
       rho1 =rho0/(2./(const%gamma+1.)/M0**2+(const%gamma-1.)/(const%gamma+1.))
       p1   =p0*(2.*const%gamma/(const%gamma+1.)*M0**2-(const%gamma-1.)/(const%gamma+1.))
       u1   =u0*rho0/rho1
      
       CALL RANDOM_SEED()
       pvar(:,:,:) = 0.
       ! Initialize upstream region
          DO j=0,MESH%JMAX+1
             DO i=0,MESH%IMAX/2
                   CALL RANDOM_NUMBER(RandNr)
                   pvar(i,j,1) = rho0 + 1e-6*(RandNr-0.5)
                   CALL RANDOM_NUMBER(RandNr)
                   pvar(i,j,2) = u0   + 1e-6*(RandNr-0.5)
                   CALL RANDOM_NUMBER(RandNr)
                   pvar(i,j,4) = p0   + 1e-6*(RandNr-0.5)
             ENDDO
          ENDDO

       ! Initialize downstream region
          DO j=0,MESH%JMAX+1
             DO i=MESH%IMAX/2+1,MESH%IMAX+1
                CALL RANDOM_NUMBER(RandNr)
                pvar(i,j,1) = rho1 + 1e-6*(RandNr-0.5)
                CALL RANDOM_NUMBER(RandNr)
                pvar(i,j,2) = u1   + 1e-6*(RandNr-0.5)
                CALL RANDOM_NUMBER(RandNr)
                pvar(i,j,4) = p1   + 1e-6*(RandNr-0.5)
             ENDDO
          ENDDO




    CASE DEFAULT
       PRINT *, 'ERROR InitialCondition not known!'
       STOP
    END SELECT

  END SUBROUTINE InitialCondition


END MODULE InitialCondition_mod
