!*****************************************************************************!
!* 
!*      $RCSfile: FluxCalculation.f90,v $
!*
!*      $Revision: 1.9 $
!*
!*      $Date: 2004/02/09 11:57:22 $
!*
!*      $Author: roller $
!*
!*****************************************************************************!

MODULE FluxCalculation_mod
  !---------------------------------------------------------------------------!
  IMPLICIT NONE
  PRIVATE
  !---------------------------------------------------------------------------!
  INTERFACE FluxCalculation
     MODULE PROCEDURE FluxCalculation
  END INTERFACE
  !---------------------------------------------------------------------------!
  PUBLIC  :: FluxCalculation
  !---------------------------------------------------------------------------!

CONTAINS

  SUBROUTINE FluxCalculation(iter,dt,xFlux,yFlux,xPrimLR,yPrimLR,Const,Mesh,MPI)
    USE Types
    USE flux_godunov_mod
    USE flux_roe_mod
    USE flux_hlle_mod
    USE flux_lax_friedrichs_mod
    USE flux_steger_warming_mod
    USE flux_central_mod
    !---------------------------------------------------------------------------!
    IMPLICIT NONE
    !---------------------------------------------------------------------------!
    ! Argument list declaration
    TYPE(tConstants)        :: Const
    TYPE(tMesh)             :: Mesh
    TYPE(tMPI)              :: MPI
    REAL                    :: dt
    REAL                    :: xFlux(0:Mesh%IMAX,0:Mesh%JMAX,Const%nvar)
    REAL                    :: yFlux(0:Mesh%IMAX,0:Mesh%JMAX,Const%nvar)
    REAL                    :: xPrimLR(2,0:Mesh%IMAX+1,0:Mesh%JMAX+1,Const%nvar)
    REAL                    :: yPrimLR(2,0:Mesh%IMAX+1,0:Mesh%JMAX+1,Const%nvar)
    INTEGER                 :: iter
    ! Local variable declaration
    INTEGER                 :: i,j
    REAL                    :: vn_l, vn_r, vt_l, vt_r
    REAL                    :: rho_l,rho_r,u_l,u_r,v_l,v_r,p_l,p_r   
    REAL                    :: flux_side(Const%nvar)
    !---------------------------------------------------------------------------!
    INTENT(IN)              :: xPrimLR, yPrimLR, Const, Mesh
    INTENT(OUT)             :: xFlux, yFlux
    !---------------------------------------------------------------------------!

    if ((iter.eq.1).and.(MPI%myrank==0)) then
       SELECT CASE (Const%iFlux)
       CASE (1)
          write(*,*) 'Flux Calculation using Godunov scheme'
       CASE (2)
          write(*,*) 'Flux Calculation using Roe scheme'
       CASE (3)
          write(*,*) 'Flux Calculation using HLLE scheme'
       CASE (4)
          write(*,*) 'Flux Calculation using Lax-Friedrichs scheme'
       CASE (5)
          write(*,*) 'Flux Calculation using Steger-Warming scheme'
       CASE (6)
          write(*,*) 'Flux Calculation using central scheme'
       CASE DEFAULt
          write(*,*) 'ERROR: no other flux function known'
          write(*,*) '       choose 1 for Godunov scheme'
          write(*,*) '       choose 2 for Roe scheme'
          write(*,*) '       choose 3 for HLLE scheme'
          write(*,*) '       choose 4 for Lax-Friedrichs scheme'
          write(*,*) '       choose 5 for Steger-Warming scheme'
          write(*,*) '       choose 6 for central scheme'
          stop
       ENDSELECT
    endif

    ! Calculate Fluxes in x direction
    DO i=0, Mesh%IMAX
       DO j=1, Mesh%JMAX
          ! Extracting left and right states in x direction
          rho_l = 
          u_l   = 
          v_l   = 
          p_l   = 
        
          rho_r = 
          u_r   = 
          v_r   = 
          p_r   = 
          
          ! Rotation in local coordinate system (trivial)
          vn_l = 
          vt_l = 
          vn_r = 
          vt_r = 
          
          SELECT CASE (Const%iFlux)
          CASE (1)
             CALL flux_godunov( Const%gamma, &
                  rho_l, rho_r, vn_l , vn_r, &
                  vt_l , vt_r , p_l  , p_r , &
                  flux_side                   )
          CASE (2)
             CALL flux_roe( Const%gamma, &
                  rho_l, rho_r, vn_l , vn_r, &
                  vt_l , vt_r , p_l  , p_r , &
                  flux_side                   )
          CASE (3)
             CALL flux_hlle( Const%gamma, &
                  rho_l, rho_r, vn_l , vn_r, &
                  vt_l , vt_r , p_l  , p_r , &
                  flux_side                   )
          CASE (4)
             CALL flux_lax_friedrichs( Const%gamma, Mesh%lmin, dt, &
                  rho_l, rho_r, vn_l , vn_r, &
                  vt_l , vt_r , p_l  , p_r , &
                  flux_side                   )
          CASE (5)
             CALL flux_steger_warming( Const%gamma, &
                  rho_l, rho_r, vn_l , vn_r, &
                  vt_l , vt_r , p_l  , p_r , &
                  flux_side                   )
          CASE (6)
             CALL flux_central( Const%gamma, Mesh%dx, dt, &
                  rho_l, rho_r, vn_l , vn_r, &
                  vt_l , vt_r , p_l  , p_r , &
                  flux_side                   )
          CASE DEFAULt
             write(*,*) 'ERROR: no other flux function known'
             write(*,*) '       choose 1 for Godunov scheme'
             write(*,*) '       choose 2 for Roe scheme'
             write(*,*) '       choose 3 for HLLE scheme'
             write(*,*) '       choose 4 for Lax-Friedrichs scheme'
             write(*,*) '       choose 5 for Steger-Warming scheme'
             write(*,*) '       choose 6 for central scheme'
             stop
          ENDSELECT

          ! Backrotation to global coordinate system (trivial)
          xFlux(i,j,:) = 
          
       ENDDO
    ENDDO

    ! Calculate Fluxes in y direction
    DO i=1, Mesh%IMAX
       DO j=0, Mesh%JMAX
          ! Extracting left and right states in y direction
          rho_l = 
          u_l   = 
          v_l   = 
          p_l   = 
        
          rho_r = 
          u_r   = 
          v_r   = 
          p_r   = 
          
          ! Rotation in local coordinate system
          vn_l = 
          vt_l = 
          vn_r = 
          vt_r = 
          
          SELECT CASE (Const%iFlux)
          CASE (1)
             CALL flux_godunov( Const%gamma, &
                  rho_l, rho_r, vn_l , vn_r, &
                  vt_l , vt_r , p_l  , p_r , &
                  flux_side                   )
          CASE (2)
             CALL flux_roe( Const%gamma, &
                  rho_l, rho_r, vn_l , vn_r, &
                  vt_l , vt_r , p_l  , p_r , &
                  flux_side                   )
          CASE (3)
             CALL flux_hlle( Const%gamma, &
                  rho_l, rho_r, vn_l , vn_r, &
                  vt_l , vt_r , p_l  , p_r , &
                  flux_side                   )
          CASE (4)
             CALL flux_lax_friedrichs( Const%gamma, Mesh%lmin, dt, &
                  rho_l, rho_r, vn_l , vn_r, &
                  vt_l , vt_r , p_l  , p_r , &
                  flux_side                   )
          CASE (5)
             CALL flux_steger_warming( Const%gamma, &
                  rho_l, rho_r, vn_l , vn_r, &
                  vt_l , vt_r , p_l  , p_r , &
                  flux_side                   )
          CASE (6)
             CALL flux_central( Const%gamma, Mesh%dy, dt, &
                  rho_l, rho_r, vn_l , vn_r, &
                  vt_l , vt_r , p_l  , p_r , &
                  flux_side                   )
          CASE DEFAULt
             write(*,*) 'ERROR: no other flux function known'
             write(*,*) '       choose 1 for Godunov scheme'
             write(*,*) '       choose 2 for Roe scheme'
             write(*,*) '       choose 3 for HLLE scheme'
             write(*,*) '       choose 4 for Lax-Friedrichs scheme'
             write(*,*) '       choose 5 for Steger-Warming scheme'
             write(*,*) '       choose 6 for central scheme'
             stop
          ENDSELECT

 
          ! Backrotation to global coordinate system
          yFlux(i,j,1) = 
          yFlux(i,j,2) = 
          yFlux(i,j,3) = 
          yFlux(i,j,4) = 
         
       ENDDO
    ENDDO

       
  END SUBROUTINE FluxCalculation


END MODULE FluxCalculation_mod
