!*****************************************************************************!
!* 
!*      $RCSfile: SetBoundaryCondition.F90,v $
!*
!*      $Revision: 1.6 $
!*
!*      $Date: 2004/02/09 18:32:56 $
!*
!*      $Author: roller $
!*
!*****************************************************************************!

MODULE SetBoundaryCondition_mod
  !---------------------------------------------------------------------------!
  IMPLICIT NONE
  PRIVATE
  !---------------------------------------------------------------------------!
  INTERFACE IniBoundaryCondition
     MODULE PROCEDURE IniBoundaryCondition
  END INTERFACE
  INTERFACE SetBoundaryCondition
     MODULE PROCEDURE SetBoundaryCondition
  END INTERFACE
  INTERFACE CloseBoundaryCondition
     MODULE PROCEDURE CloseBoundaryCondition
  END INTERFACE

  REAL,POINTER            :: sendleft(:,:), sendright(:,:)  ! Randwerte-Vektor
  REAL,POINTER            :: sendup(:,:), senddown(:,:)     ! Randwerte-Vektor
  REAL,POINTER            :: getleft(:,:), getright(:,:)    ! Randwerte-Vektor
  REAL,POINTER            :: getupper(:,:), getlower(:,:)   ! Randwerte-Vektor
  INTEGER                 :: allocStat

  !---------------------------------------------------------------------------!
  PUBLIC  :: SetBoundaryCondition,IniBoundaryCondition,CloseBoundaryCondition
  !---------------------------------------------------------------------------!

CONTAINS

  SUBROUTINE SetBoundaryCondition(pvar,Const,Mesh,MPI)
    USE Types
    !---------------------------------------------------------------------------!
    IMPLICIT NONE
    !---------------------------------------------------------------------------!



    !---------------------------------------------------------------------------!
    ! Argument list declaration
    TYPE(tConstants)        :: Const
    TYPE(tMesh)             :: Mesh
    TYPE(tMPI)              :: MPI
    REAL                    :: pvar(0:Mesh%IMAX+1,0:Mesh%JMAX+1,Const%nvar)
    ! Local variable declaration





    !---------------------------------------------------------------------------!
    INTENT(IN)              :: Const, Mesh
    INTENT(INOUT)           :: pvar
    !---------------------------------------------------------------------------!

    ! Implemented Boundary Conditions:
    ! Const%BoundCond = 1: Extrapolation 1st order at all boundaries
    ! Const%BoundCond = 2: Extrapolation 1st order in x, periodic in y-direction
    ! Const%BoundCond = 3: Periodic in x-direction, extrapolation 1st order in y
    ! Const%BoundCond = 4: Periodic in x- and y-direction


    ! Set left boundary


       IF ((Const%BoundCond.eq.1).or.(Const%BoundCond.eq.2)) THEN
          pvar(0,:,:) = pvar(1,:,:)                       ! Extrapolation 1st order 
       ELSEIF ((Const%BoundCond.eq.3).or.(Const%BoundCond.eq.4)) THEN
          pvar(0,:,:) = pvar(MPI%IMAX,:,:)               ! Periodic BC in x-direction
       ELSE
          write(*,*) 'No other boundary condition known'
          STOP
       ENDIF


    ! Set right boundary


    IF ((Const%BoundCond.eq.1).or.(Const%BoundCond.eq.2)) THEN
       pvar(Mesh%IMAX+1,:,:) =                         ! Extrapolation 1st order 
    ELSEIF ((Const%BoundCond.eq.3).or.(Const%BoundCond.eq.4)) THEN
       pvar(Mesh%IMAX+1,:,:) =                         ! Periodic BC in x-direction
    ELSE
       write(*,*) 'No other boundary condition known'
       STOP
    ENDIF


    ! Set lower boundary


    IF ((Const%BoundCond.eq.1).or.(Const%BoundCond.eq.3)) THEN
       pvar(:,0,:) =                                   ! Extrapolation 1st order 
    ELSEIF ((Const%BoundCond.eq.2).or.(Const%BoundCond.eq.4)) THEN
       pvar(:,0,:) =                                   ! Periodic BC in y-direction
    ELSE
       write(*,*) 'No other boundary condition known'
       STOP
    ENDIF


    ! Set upper boundary


    IF ((Const%BoundCond.eq.1).or.(Const%BoundCond.eq.3)) THEN
       pvar(:,Mesh%JMAX+1,:) =                         ! Extrapolation 1st order 
    ELSEIF ((Const%BoundCond.eq.2).or.(Const%BoundCond.eq.4)) THEN
       pvar(:,Mesh%JMAX+1,:) =                         ! Periodic BC in y-direction
    ELSE
       write(*,*) 'No other boundary condition known'
       STOP
    ENDIF

  
  END SUBROUTINE SetBoundaryCondition


  SUBROUTINE IniBoundaryCondition(Const,Mesh)
    USE Types
    !---------------------------------------------------------------------------!
    IMPLICIT NONE
    !---------------------------------------------------------------------------!
    ! Argument list declaration
    TYPE(tConstants)        :: Const
    TYPE(tMesh)             :: Mesh
    !---------------------------------------------------------------------------!
    INTENT(IN)              :: Const, Mesh
    !---------------------------------------------------------------------------!



  END SUBROUTINE IniBoundaryCondition

  SUBROUTINE CloseBoundaryCondition()
    USE Types
    !---------------------------------------------------------------------------!
    IMPLICIT NONE
    !---------------------------------------------------------------------------!
    ! Argument list declaration
    ! Local variable declaration
    !---------------------------------------------------------------------------!


  END SUBROUTINE CloseBoundaryCondition

END MODULE SetBoundaryCondition_mod
