!*****************************************************************************!
!* 
!*      $RCSfile: Flux_central.f90,v $
!*
!*      $Revision: 1.3 $
!*
!*      $Date: 2003/01/15 07:35:10 $
!*
!*      $Author: roller $
!*
!*****************************************************************************!

MODULE flux_central_mod
  !--------------------------------------------------------------------------
  IMPLICIT NONE
  PRIVATE
  !----------------------------------------------------------------------------
  INTERFACE flux_central
     MODULE PROCEDURE flux_central
  END INTERFACE
  !----------------------------------------------------------------------------

  PUBLIC  :: flux_central

CONTAINS

  SUBROUTINE flux_central( gamma, dx, dt,             &
                           rho_l, rho_r, vn_l , vn_r, &
                           vt_l , vt_r , p_l  , p_r , &
                           flux_side                   )

    !--------------------------------------------------------------------------
    IMPLICIT NONE
    !--------------------------------------------------------------------------

    !argument list declaration
    REAL         :: gamma, dx, dy, dt
    REAL         :: rho_l, rho_r
    REAL         :: vn_l , vn_r
    REAL         :: vt_l , vt_r
    REAL         :: p_l  , p_r
    REAL         :: flux_side(4)

    !local variable declaration
    REAL         :: e_r,e_l
    REAL         :: f1_r(4),f1_l(4)
    !--------------------------------------------------------------------------
    INTENT(IN)   :: gamma, dx, dt,             &
                    rho_l, rho_r, vn_l , vn_r, &
                    vt_l , vt_r , p_l  , p_r 
    INTENT(OUT)  :: flux_side    
    !--------------------------------------------------------------------------

    ! Calculate left/right energy 
    
    e_r=1./(gamma-1.)*p_r+0.5*rho_r*(vn_r*vn_r+vt_r*vt_r)
    e_l=1./(gamma-1.)*p_l+0.5*rho_l*(vn_l*vn_l+vt_l*vt_l)
    
    ! Calculalte the physical fluxes
    
    f1_r(1) = rho_r*vn_r
    f1_r(2) = rho_r*vn_r*vn_r + p_r
    f1_r(3) = rho_r*vn_r*vt_r
    f1_r(4) = vn_r*(e_r + p_r)
    
    f1_l(1) = rho_l*vn_l
    f1_l(2) = rho_l*vn_l*vn_l + p_l
    f1_l(3) = rho_l*vn_l*vt_l
    f1_l(4) = vn_l*(e_l + p_l)
    
    ! Calculate Roe fluxes
    
    flux_side(:) = 0.5*( f1_r(:)+f1_l(:) ) 
    
    RETURN
  END SUBROUTINE flux_central
  
END MODULE flux_central_mod
