!*****************************************************************************!
!* 
!*      $RCSfile: Flux_hlle.f90,v $
!*
!*      $Revision: 1.5 $
!*
!*      $Date: 2004/01/29 11:10:54 $
!*
!*      $Author: iagmidu $
!*
!*****************************************************************************!

MODULE flux_hlle_mod
  !--------------------------------------------------------------------------
  IMPLICIT NONE
  PRIVATE
  !----------------------------------------------------------------------------
  INTERFACE flux_hlle
     MODULE PROCEDURE flux_hlle
  END INTERFACE
  !----------------------------------------------------------------------------

  PUBLIC  :: flux_hlle

CONTAINS

  SUBROUTINE flux_hlle( gamma,                     &
                           rho_l, rho_r, vn_l , vn_r, &
                           vt_l , vt_r , p_l  , p_r , &
                           flux_side                   )

    !--------------------------------------------------------------------------
    IMPLICIT NONE


    !argument list declaration
    REAL         :: gamma
    REAL         :: rho_l, rho_r
    REAL         :: vn_l , vn_r
    REAL         :: vt_l , vt_r
    REAL         :: p_l  , p_r
    REAL         :: flux_side(4)

    !local variable declaration
    REAL         :: fl(4),fr(4),ul(4),ur(4)
    REAL         :: c_l,c_r,H_l,H_r,kin
    REAL         :: um, vm, Hm, cm
    REAL         :: arp, alm
    !--------------------------------------------------------------------------
    INTENT(IN)   :: gamma,                     &
                    rho_l, rho_r, vn_l , vn_r, &
                    vt_l , vt_r , p_l  , p_r 
    INTENT(OUT)  :: flux_side    
    !--------------------------------------------------------------------------
    ! Flux in left cell
    fl(1) = rho_l*vn_l
    fl(2) = rho_l*vn_l*vn_l+p_l
    fl(3) = rho_l*vn_l*vt_l
    fl(4) =       vn_l*(gamma/(gamma-1.)*p_l+rho_l/2.*(vn_l*vn_l+vt_l*vt_l))
    ! Flux in right cell
    fr(1) = rho_r*vn_r
    fr(2) = rho_r*vn_r*vn_r+p_r
    fr(3) = rho_r*vn_r*vt_r
    fr(4) =       vn_r*(gamma/(gamma-1.)*p_r+rho_r/2.*(vn_r*vn_r+vt_r*vt_r))
    ! Left conservative state vector
    ul(1) = rho_l
    ul(2) = rho_l*vn_l
    ul(3) = rho_l*vt_l
    ul(4) = p_l/(gamma-1.)+0.5*rho_l*(vn_l*vn_l+vt_l*vt_l)
    ! Right conservative state vector
    ur(1) = rho_r
    ur(2) = rho_r*vn_r
    ur(3) = rho_r*vt_r
    ur(4) = p_r/(gamma-1.)+0.5*rho_r*(vn_r*vn_r+vt_r*vt_r)
    ! Calculation of sound speeds
    c_l=SQRT(gamma*p_l/rho_l)
    c_r=SQRT(gamma*p_r/rho_r)
    ! Left enthalpy
    kin = 0.5*(vn_l*vn_l+vt_l*vt_l)             ! left kinetic energy
    H_l = kin + gamma/(gamma-1.)*p_l/rho_l      ! the left enthalpy
    ! Right enthalpy
    kin = 0.5*(vn_r*vn_r+vt_r*vt_r)             ! right kinetic energy
    H_r = kin + gamma/(gamma-1.)*p_r/rho_r      ! the right enthalpy
    ! Roe mean values
    um=(SQRT(rho_r)*vn_r+SQRT(rho_l)*vn_l)/(SQRT(rho_r)+SQRT(rho_l))
    vm=(SQRT(rho_r)*vt_r+SQRT(rho_l)*vt_l)/(SQRT(rho_r)+SQRT(rho_l))
    Hm=(SQRT(rho_r)*H_r +SQRT(rho_l)*H_l )/(SQRT(rho_r)+SQRT(rho_l))
    cm=SQRT((gamma-1.)*(Hm-0.5*(um*um+vm*vm)))
    ! Signal speeds
    arp=MAX(vn_r+c_r,um+cm,0.)
    alm=MIN(vn_l-c_l,um-cm,0.)
    ! HLLE-Flux
    flux_side(:)=(arp*fl(:)-alm*fr(:))/(arp-alm)+arp*alm/(arp-alm)*(ur(:)-ul(:))
    RETURN
  END SUBROUTINE flux_hlle

END MODULE flux_hlle_mod
