!*****************************************************************************!
!* 
!*      $RCSfile: SetBoundaryConditionLR.F90,v $
!*
!*      $Revision: 1.3 $
!*
!*      $Date: 2004/02/09 18:32:56 $
!*
!*      $Author: roller $
!*
!*****************************************************************************!

MODULE SetBoundaryConditionLR_mod
  !---------------------------------------------------------------------------!
  IMPLICIT NONE
  PRIVATE
  !---------------------------------------------------------------------------!
  INTERFACE IniBoundaryConditionLR
     MODULE PROCEDURE IniBoundaryConditionLR
  END INTERFACE
  INTERFACE SetBoundaryConditionLR
     MODULE PROCEDURE SetBoundaryConditionLR
  END INTERFACE
  INTERFACE CloseBoundaryConditionLR
     MODULE PROCEDURE CloseBoundaryConditionLR
  END INTERFACE

  REAL,POINTER            :: sendleft(:,:,:), sendright(:,:,:)  ! Randwerte-Vektor
  REAL,POINTER            :: sendup(:,:,:), senddown(:,:,:)     ! Randwerte-Vektor
  REAL,POINTER            :: getleft(:,:,:), getright(:,:,:)    ! Randwerte-Vektor
  REAL,POINTER            :: getupper(:,:,:), getlower(:,:,:)   ! Randwerte-Vektor
  INTEGER                 :: allocStat

  !---------------------------------------------------------------------------!
  PUBLIC  :: SetBoundaryConditionLR,IniBoundaryConditionLR,CloseBoundaryConditionLR
  !---------------------------------------------------------------------------!

CONTAINS

  SUBROUTINE SetBoundaryConditionLR(state,Const,Mesh,MPI)
    USE Types
    !---------------------------------------------------------------------------!
    IMPLICIT NONE
    !---------------------------------------------------------------------------!
#ifdef parallel
    include 'mpif.h'
#endif
    !---------------------------------------------------------------------------!
    ! Argument list declaration
    TYPE(tConstants)        :: Const
    TYPE(tMesh)             :: Mesh
    TYPE(tMPI)              :: MPI
    REAL                    :: state(2,0:Mesh%IMAX+1,0:Mesh%JMAX+1,Const%nvar)
    ! Local variable declaration
#ifdef parallel
    integer                 :: left, right                    ! Nachbarprozesse
    integer                 :: rqleft,rqright,rqupper,rqlower ! Request-Handles
    integer                 :: status(MPI_STATUS_SIZE)
#endif
    !---------------------------------------------------------------------------!
    INTENT(IN)              :: Const, Mesh
    INTENT(INOUT)           :: state
    !---------------------------------------------------------------------------!

    ! Implemented Boundary Conditions:
    ! Const%BoundCond = 1: Extrapolation 1st order at all boundaries
    ! Const%BoundCond = 2: Extrapolation 1st order in x, periodic in y-direction
    ! Const%BoundCond = 3: Periodic in x-direction, extrapolation 1st order in y
    ! Const%BoundCond = 4: Periodic in x- and y-direction


    ! Set left boundary

#ifdef parallel
    call mpi_cart_shift(MPI%comm_cart,0,1,left,right,MPI%ierror)

    ! Umkopieren: sendright = Vektor der letzen inneren Spalte
    sendright(:,1:Mesh%JMAX,:) = state(:,Mesh%IMAX,1:Mesh%JMAX,:)

    ! Sende sendright an den rechten Nachbarn
    ! Bin ich am rechten Rechengebietsrand und RB=periodisch,
    ! dann ist mein rechter Nachbar der am linken Rechengebietsrand
    ! Bin ich am rechten Rechengebietsrand und RB=Extrapolation,
    ! dann sendet das ins Leere (nach MPI_PROC_NULL). Das strt nicht.

    call mpi_issend(sendright,2*Mesh%JMAX*Const%nvar,mpi_real,right,17, &
         MPI%comm_cart,rqright,MPI%ierror)

    ! Empfange vom linken Nachbarn
    ! Bin ich am linken Rechengebietsrand und RB=periodisch,
    ! dann ist das der Prozess am rechten Rechengebietsrand
    ! Bin ich am linken Rechengebietsrand und RB=Extrapolation,
    ! dann empfange ich aus dem Nichts (aus MPI_PROC_NULL). Strt auch nicht.
    
    call mpi_recv(getleft,2*Mesh%JMAX*Const%nvar,mpi_real,left,17, &
         MPI%comm_cart,status,MPI%ierror)
    call mpi_wait(rqright,status,MPI%ierror)

    ! Zurckspeichern:
    state(:,0,1:Mesh%JMAX,:) = getleft(:,1:Mesh%JMAX,:)
    
    ! Bin ich echter Rand und RB=Extrapolation, 
    ! dann mu ich jetzt noch das Richtige setzen:

    if (MPI%coords(1) == 0) then
       IF ((Const%BoundCond.eq.1).or.(Const%BoundCond.eq.2)) THEN
          state(:,0,:,:) = state(:,1,:,:)                       ! Extrapolation 1st order 
       endif
    endif

#else
       IF ((Const%BoundCond.eq.1).or.(Const%BoundCond.eq.2)) THEN
          state(:,0,:,:) = state(:,1,:,:)                       ! Extrapolation 1st order 
       ELSEIF ((Const%BoundCond.eq.3).or.(Const%BoundCond.eq.4)) THEN
          state(:,0,:,:) = state(:,MPI%IMAX,:,:)               ! Periodic BC in x-direction
       ELSE
          write(*,*) 'No other boundary condition known'
          STOP
       ENDIF
#endif

    ! Set right boundary

#ifdef parallel
    ! Umkopieren: sendleft = Vektor der ersten inneren Spalte
    sendleft(:,1:Mesh%JMAX,:) = state(:,1,1:Mesh%JMAX,:)

    ! Sende sendleft an den linken Nachbarn

    call mpi_issend(sendleft,2*Mesh%JMAX*Const%nvar,mpi_real,left,18, &
         MPI%comm_cart,rqleft,MPI%ierror)

    ! Empfange vom linken Nachbarn
    ! Bin ich am linken Rechengebietsrand und RB=periodisch,
    ! dann ist das der Prozess am rechten Rechengebietsrand
    ! Bin ich am linken Rechengebietsrand und RB=Extrapolation,
    ! dann empfange ich aus dem Nichts (aus MPI_PROC_NULL). Strt auch nicht.
    
    call mpi_recv(getright,2*Mesh%JMAX*Const%nvar,mpi_real,right,18, &
         MPI%comm_cart,status,MPI%ierror)
    call mpi_wait(rqleft,status,MPI%ierror)

    ! Zurckspeichern:
    state(:,Mesh%IMAX+1,1:Mesh%JMAX,:) = getright(:,1:Mesh%JMAX,:)
    
    ! Bin ich echter Rand und RB=Extrapolation, 
    ! dann mu ich jetzt noch das Richtige setzen:

    if (MPI%coords(1) == (MPI%dims(1)-1)) then
       IF ((Const%BoundCond.eq.1).or.(Const%BoundCond.eq.2)) THEN
          state(:,Mesh%IMAX+1,:,:) = state(:,Mesh%IMAX,:,:)     ! Extrapolation 1st order 
       endif
    endif

#else
    IF ((Const%BoundCond.eq.1).or.(Const%BoundCond.eq.2)) THEN
       state(:,Mesh%IMAX+1,:,:) = state(:,Mesh%IMAX,:,:)     ! Extrapolation 1st order 
    ELSEIF ((Const%BoundCond.eq.3).or.(Const%BoundCond.eq.4)) THEN
       state(:,Mesh%IMAX+1,:,:) = state(:,1,:,:)             ! Periodic BC in x-direction
    ELSE
       write(*,*) 'No other boundary condition known'
       STOP
    ENDIF
#endif

    ! Set lower boundary

#ifdef parallel
    call mpi_cart_shift(MPI%comm_cart,1,1,left,right,MPI%ierror)

    ! Umkopieren: sendup = Vektor der letzen inneren Zeile
    sendup(:,1:Mesh%IMAX,:) = state(:,1:Mesh%IMAX,Mesh%JMAX,:)

    ! Sende sendup an den oberen Nachbarn

    call mpi_issend(sendup,2*Mesh%IMAX*Const%nvar,mpi_real,right,19, &
         MPI%comm_cart,rqupper,MPI%ierror)

    ! Empfange vom unteren Nachbarn
    
    call mpi_recv(getlower,2*Mesh%IMAX*Const%nvar,mpi_real,left,19, &
         MPI%comm_cart,status,MPI%ierror)
    call mpi_wait(rqupper,status,MPI%ierror)

    ! Zurckspeichern:
    state(:,1:Mesh%IMAX,0,:) = getlower(:,1:Mesh%IMAX,:)
    
    ! Bin ich echter Rand und RB=Extrapolation, 
    ! dann mu ich jetzt noch das Richtige setzen:

    if (MPI%coords(2) == 0) then
       IF ((Const%BoundCond.eq.1).or.(Const%BoundCond.eq.3)) THEN
          state(:,:,0,:) = state(:,:,1,:)                       ! Extrapolation 1st order 
       endif
    endif

#else
    IF ((Const%BoundCond.eq.1).or.(Const%BoundCond.eq.3)) THEN
       state(:,:,0,:) = state(:,:,1,:)                       ! Extrapolation 1st order 
    ELSEIF ((Const%BoundCond.eq.2).or.(Const%BoundCond.eq.4)) THEN
       state(:,:,0,:) = state(:,:,MESH%JMAX,:)               ! Periodic BC in y-direction
    ELSE
       write(*,*) 'No other boundary condition known'
       STOP
    ENDIF
#endif

    ! Set upper boundary

#ifdef parallel
    ! Umkopieren: senddown = Vektor der ersten inneren Zeile
    senddown(:,1:Mesh%IMAX,:) = state(:,1:Mesh%IMAX,1,:)

    ! Sende sendup an den unteren Nachbarn

    call mpi_issend(senddown,2*Mesh%IMAX*Const%nvar,mpi_real,left,20, &
         MPI%comm_cart,rqlower,MPI%ierror)

    ! Empfange vom unteren Nachbarn
    
    call mpi_recv(getupper,2*Mesh%IMAX*Const%nvar,mpi_real,right,20, &
         MPI%comm_cart,status,MPI%ierror)
    call mpi_wait(rqlower,status,MPI%ierror)

    ! Zurckspeichern:
    state(:,1:Mesh%IMAX,Mesh%JMAX+1,:) = getupper(:,1:Mesh%IMAX,:)
    
    ! Bin ich echter Rand und RB=Extrapolation, 
    ! dann mu ich jetzt noch das Richtige setzen:

    if (MPI%coords(2) == (MPI%dims(2)-1)) then
       IF ((Const%BoundCond.eq.1).or.(Const%BoundCond.eq.3)) THEN
          state(:,:,Mesh%JMAX+1,:) = state(:,:,Mesh%JMAX,:)     ! Extrapolation 1st order 
       endif
    endif

#else
    IF ((Const%BoundCond.eq.1).or.(Const%BoundCond.eq.3)) THEN
       state(:,:,Mesh%JMAX+1,:) = state(:,:,Mesh%JMAX,:)     ! Extrapolation 1st order 
    ELSEIF ((Const%BoundCond.eq.2).or.(Const%BoundCond.eq.4)) THEN
       state(:,:,Mesh%JMAX+1,:) = state(:,:,1,:)             ! Periodic BC in y-direction
    ELSE
       write(*,*) 'No other boundary condition known'
       STOP
    ENDIF
#endif
    
  END SUBROUTINE SetBoundaryConditionLR


  SUBROUTINE IniBoundaryConditionLR(Const,Mesh)
    USE Types
    !---------------------------------------------------------------------------!
    IMPLICIT NONE
    !---------------------------------------------------------------------------!
    ! Argument list declaration
    TYPE(tConstants)        :: Const
    TYPE(tMesh)             :: Mesh
    !---------------------------------------------------------------------------!
    INTENT(IN)              :: Const, Mesh
    !---------------------------------------------------------------------------!

#ifdef parallel
    ALLOCATE(sendleft(2,1:Mesh%JMAX,Const%nvar),   &
             sendright(2,1:Mesh%JMAX,Const%nvar),  &
             sendup(2,1:Mesh%IMAX,Const%nvar),   &
             senddown(2,1:Mesh%IMAX,Const%nvar),  &
             getleft(2,1:Mesh%JMAX,Const%nvar),   &
             getright(2,1:Mesh%JMAX,Const%nvar),  &
             getupper(2,1:Mesh%IMAX,Const%nvar),   &
             getlower(2,1:Mesh%IMAX,Const%nvar),  &
             STAT = allocStat )
    IF(allocStat.NE.0) THEN
       PRINT *, 'ERROR IniBoundaryConditionLR: ', &
                'Could not allocate all variables. '
       STOP
    ENDIF

    getleft  = 0.
    getright = 0.
    getupper = 0.
    getlower = 0.
#endif

  END SUBROUTINE IniBoundaryConditionLR

  SUBROUTINE CloseBoundaryConditionLR()
    USE Types
    !---------------------------------------------------------------------------!
    IMPLICIT NONE
    !---------------------------------------------------------------------------!
    ! Argument list declaration
    ! Local variable declaration
    !---------------------------------------------------------------------------!

#ifdef parallel
    DEALLOCATE(sendleft, sendright, getleft, getright,  &
             STAT = allocStat )
    IF(allocStat.NE.0) THEN
       PRINT *, 'ERROR CloseBoundaryConditionLR: ', &
                'Could not deallocate all variables. '
       STOP
    ENDIF
#endif
  END SUBROUTINE CloseBoundaryConditionLR

END MODULE SetBoundaryConditionLR_mod
